--[[
************************************************************************
Project				: SKG
Author				: zhinjio
Project Revision	: r158
Project Date		: 20090521034504

File				: SKG_GUI_Main.lua
Commit Author		: zhinjio
Commit Revision		: 158
Commit Date			: 20090521034504
************************************************************************
Description	:
	This file contains the definition of the main frame, and all functions
	pertaining to its use

TODO		:

************************************************************************
(see bottom of file for changelog)
************************************************************************
--]]
local SKG = LibStub( "AceAddon-3.0"):GetAddon( "SKG" )
local SKGL = LibStub( "AceLocale-3.0"):GetLocale( "SKG" )

-- Function for switching help text
function SKG:SwitchActiveList( value )
	--[[ Removed for LQTC conversion
	Dewdrop:Close()
	]]--
	-- Remove the old selected texture, if there was someone selected
	if ( SKG_ScrollFrame.selected ~= 0 ) then
		local i = SKG_ScrollFrame.selected - FauxScrollFrame_GetOffset( SKG_ScrollFrame )
		if ( ( i > 0 ) and ( i <= SKG_maxVisibleListItems ) ) then
			SKG_ListItem[i]:SetNormalTexture( "" )
			SKG_ListItem[i]:SetHighlightTexture( "" )
		end
		SKG_ScrollFrame.selected = 0
		SKG.selectedPID = 0
	end

	-- If there were any synckeys shown on the SyncStatus page, hide 'em.
	for i = 1, SKG_maxVisibleSYFListItems do
		SKG_SYF_NameListItem[i]:Hide()
		SKG_SYF_HashListItem[i]:Hide()
		SKG_currentSyncItems = 0
	end

	-- We need to behave differently if the list being selected is "Players"
	-- If it is, the List Mgmt pane should only allow "Insert" and "Remove" functions
	if ( value == "Players" ) then
		SKG_LF_DelListButton:Disable()
		SKG_LF_RenListButton:Disable()
		SKG_LF_CloneListButton:Disable()
		SKG_LF_SnapshotListButton:Disable()
		SKG_LF_BroadcastListButton:Enable()
		SKG_LF_SpamListButton:Disable()
		SKG_LF_MoveUpButton:Disable()
		SKG_LF_MoveDownButton:Disable()
		SKG_LF_SuicidePlayerButton:Disable()
		SKG_LF_ReservePlayerButton:Disable()
		-- also disable 'Auto-Add' checkbox and text
		SKG_LF_AutoAddCB:Disable()
		SKG_DisableNormalText( SKG_LF_AutoAddCBText )
	else
		SKG_LF_DelListButton:Enable()
		SKG_LF_RenListButton:Enable()
		SKG_LF_CloneListButton:Enable()
		SKG_LF_SnapshotListButton:Enable()
		SKG_LF_BroadcastListButton:Enable()
		SKG_LF_SpamListButton:Enable()
		SKG_LF_SuicidePlayerButton:Enable()
		SKG_LF_AutoAddCB:Enable()
		SKG_EnableNormalText( SKG_LF_AutoAddCBText )
		-- if we're in a raid, and "raidonly" is enabled, disable some functions
		if ( SKG.inRaid and SKG.db.profile.Options.General.raidonly ) then
			SKG_LF_MoveUpButton:Disable()
			SKG_LF_MoveDownButton:Disable()
			SKG_LF_ReservePlayerButton:Disable()
		else
			SKG_LF_MoveUpButton:Enable()
			SKG_LF_MoveDownButton:Enable()
			SKG_LF_ReservePlayerButton:Enable()
		end

		-- This is a good a place as any. If the person is an admin, and has switched to
		-- a list which has no lastmod info, inform them that they suck, and to update.
		if ( SKG_SECURITY_LEVEL ~= "U" ) then
			if ( SKG.db.profile.LastMod[value] == nil ) then
				SKG_Print( SKGL["SKG_ERROR_NOLASTMODINFO"] )
			end
		end
	end

	
	SKG_ScrollFrame.selected = 0
	SKG_ActiveListTitle:SetText( value )
	SKG_UpdateCurrentList()
end

--[[ Removing to use LQTC instead
SKG.listSelections = {
	type = "group",
	handler = SKG,
	args = { },
} ]]--

function MouseHandler_SLQTC( list )
	addon:SwitchActiveList( list )
	SelectList_QTC:Hide()
end

function SKG_UpdateLists( )
	--[[
	SKG.listSelections.args = { }
	-- Insert our "dummy" Players list, since it should always be there.
	local t = {
		type = "execute",
		name = "Players",
		desc = "Players",
		func = "SwitchActiveList",
		passValue = "Players"
	}
	tinsert( SKG.listSelections.args, t )

	for listName, rollList in pairs( SKG.db.profile.RollLists ) do
		t = {
			type = "execute",
			name = listName,
			desc = listName,
			func = "SwitchActiveList",
			passValue = listName
		}
		tinsert( SKG.listSelections.args, t )
	end ]]--
	-- switch to using LQTC methods for populating the dropdown
	SKG_SelectList_QTC:Clear()
	
	SKG_SelectList_QTC:SetCallback("OnMouseDown", MouseHandler)
    local lineNum = SelectList_QTC:AddLine()
	SKG_SelectList_QTC:SetCell( lineNum, 1, "Players", "Players" )
	for listName, rollList in pairs( SKG.db.profile.RollLists ) do
		lineNum = tooltip:AddLine()
		SKG_SelectList_QTC:SetCell( lineNum, 1, listName, listName )
	end
	
	SKG_SelectListButton:Enable() 
end

function SKG_ResetCurrentList()
	SKG_ActiveListTitle:SetText( "" )
	for i = 1,SKG_maxVisibleListItems do
		SKG_ListItem[i]:SetText( "" )
		SKG_TooltipDisplay( SKG_ListItem[i], "", 0 )
	end
end

function SKG_SetOptionSwitcherButtonTexture( value )

	local SKG_OSB_NTexture = SKG_OptionSwitcherButton:CreateTexture( "SKG_OSB_NTexture", "BACKGROUND" )
	SKG_OSB_NTexture:SetTexture( [[Interface\Addons\SKG\BLP\FunctionButton_]] .. value .. [[_Up]] )
	SKG_OSB_NTexture:SetTexCoord( 0, 1, 0, 1 )
	SKG_OSB_NTexture:SetAllPoints( SKG_OptionSwitcherButton )
	local SKG_OSB_PTexture = SKG_OptionSwitcherButton:CreateTexture( "SKG_OSB_PTexture", "BACKGROUND" )
	SKG_OSB_PTexture:SetTexture( [[Interface\Addons\SKG\BLP\FunctionButton_]] .. value .. [[_Down]] )
	SKG_OSB_PTexture:SetTexCoord( 0, 1, 0, 1 )
	SKG_OSB_PTexture:SetAllPoints( SKG_OptionSwitcherButton )
	local SKG_OSB_DTexture = SKG_OptionSwitcherButton:CreateTexture( "SKG_OSB_DTexture", "BACKGROUND" )
	SKG_OSB_DTexture:SetTexture( [[Interface\Addons\SKG\BLP\FunctionButton_]] .. value .. [[_Disabled]] )
	SKG_OSB_DTexture:SetTexCoord( 0, 1, 0, 1 )
	SKG_OSB_DTexture:SetAllPoints( SKG_OptionSwitcherButton )

	SKG_OptionSwitcherButton:SetNormalTexture( SKG_OSB_NTexture )
	SKG_OptionSwitcherButton:SetPushedTexture( SKG_OSB_PTexture )
	SKG_OptionSwitcherButton:SetDisabledTexture( SKG_OSB_DTexture )

end

function SKG_FlipFrame( this )

	-- The long term of this is going to check against your permissions and the current state
	-- for now, just toggle between existing frames:
	if ( SKG_FRAME_STATE == "Hidden" ) then
		SKG_OptionsFrame:Show()
		SKG_FRAME_STATE = "Options"
	elseif ( SKG_FRAME_STATE == "Options" ) then
		SKG_OptionsFrame:Hide()
 		SKG_HelpFrame:Show()
		SKG_FRAME_STATE = "Help"
 	elseif ( SKG_FRAME_STATE == "Help" ) then
		SKG_HelpFrame:Hide()
		SKG_BidFrame:Show()
		SKG_FRAME_STATE = "Bid"
		-- If we've got current loot items, open the BBF as well
		if ( ( SKG_SECURITY_LEVEL ~= "U" ) and ( #SKG.LootItems ~= 0 ) ) then
			SKG_BidButtonFrame:Show()
		end
	elseif ( SKG_FRAME_STATE == "Bid" ) then
		SKG_BidFrame:Hide()
		SKG_BidButtonFrame:Hide()
		if ( SKG_SECURITY_LEVEL ~= "U" ) then
			SKG_ListFrame:Show()
			SKG_FRAME_STATE = "List"
		else
			SKG_FRAME_STATE = "Hidden"
		end
	elseif ( SKG_FRAME_STATE == "List" ) then
		SKG_ListFrame:Hide()
		-- couldn't be here unless you're S/P/A
		-- so just show sync frame
		SKG_SyncFrame:Show()
		SKG_FRAME_STATE = "Sync"
	else
		-- just hide the sync frame
		SKG_SyncFrame:Hide()
		SKG_FRAME_STATE = "Hidden"
	end
end

function SKG_OnMouseDown(arg1)
   if arg1=="LeftButton" then
      this:StartMoving()
   end
end

function SKG_OnMouseUp(arg1)
   if arg1=="LeftButton" then
      this:StopMovingOrSizing()
      SKG.db.profile.Options.General.xPos = SKG_Frame:GetLeft()
      SKG.db.profile.Options.General.yPos = SKG_Frame:GetTop()
   end
end

function SKG_ScrollingList_Update()
	SKG_UpdateCurrentList()
end

function SKG_ItemButton_OnClick( index )
	-- we should only do something if the button selected by the user actually contains
	-- something. Otherwise, they're clicking past the end of the list
	local currList = SKG_ActiveListTitle:GetText()

	-- Don't do anything if we don't have a list selected
	if ( currList ) then
		SKG_Debug( "IBOC: in currlist" )
		-- Go through our current buttons until _PID is 0
		local listLen = SKG_maxVisibleListItems
		for i = 1, SKG_maxVisibleListItems do
			if ( SKG_ListItem[i]._PID == 0 ) then
				listLen = i - 1
				break
			end
		end

		SKG_Debug( "IBOC: index = " .. index .. ", listlen = " .. listLen ) 
		if ( index > listLen ) then
			-- don't do anything, we're past the end of displayed items
		else
			-- ok, we've selected a button with something on it.

			-- if we've pushed the currently selected button, deselect
			if ( SKG_ListItem[index]._PID == SKG.selectedPID ) then
				-- unselect
				SKG_ScrollFrame.selected = 0
				SKG.selectedPID = 0
				SKG_ListItem[index]:SetNormalTexture( "" )
				SKG_ListItem[index]:SetHighlightTexture( "" )
			else
				-- deselect the old selected player
				SKG_Debug( "sel = " .. SKG_ScrollFrame.selected ) 
				SKG_Debug( "off = " .. FauxScrollFrame_GetOffset( SKG_ScrollFrame ) ) 

				if ( SKG_ScrollFrame.selected ~= 0 ) then
					local i = SKG_ScrollFrame.selected - FauxScrollFrame_GetOffset( SKG_ScrollFrame )
					if ( ( i > 0 ) and ( i <= SKG_maxVisibleListItems ) ) then
						SKG_ListItem[i]:SetNormalTexture( "" )
						SKG_ListItem[i]:SetHighlightTexture( "" )
					end
				end
				-- select new one
				SKG_ScrollFrame.selected = index + FauxScrollFrame_GetOffset( SKG_ScrollFrame )
				local li = SKG_ListItem[index]
				SKG.selectedPID = li._PID
				li:SetNormalTexture( [[Interface\Addons\SKG\BLP\highlight]] )
				local nTex = li:GetNormalTexture()
				nTex:SetAllPoints()
				nTex:SetTexCoord( (30/256), (1-(10/256)), (6/32), (1-(6/32)) )
				li:SetHighlightTexture( [[Interface\Addons\SKG\BLP\highlight]] )
				local hTex = li:GetHighlightTexture()
				hTex:SetAllPoints()
				hTex:SetTexCoord( (30/256), (1-(10/256)), (6/32), (1-(6/32)) )
				-- change the text on the reserve button depending on whether the selected
				-- button player is reserved or not
				local isReserved = false
				local i = 1
				while ( ( i <= #SKG.ReservedPlayers ) and ( isReserved == false ) ) do
					if ( li._PID == SKG.ReservedPlayers[i] ) then
						isReserved = true
						break
					end
					i = i + 1
				end
				if ( isReserved == true ) then
					SKG_LF_ReservePlayerButton:SetText( SKGL["SKG_LBL_LF_UNRESERVEPLAYER_TEXT"] )
					SKG_LF_ReservePlayerButton:Enable()
				else
					SKG_LF_ReservePlayerButton:SetText( SKGL["SKG_LBL_LF_RESERVEPLAYER_TEXT"] )
					if ( ( SKG.inRaid ) and
						 ( SKG.db.profile.Options.General.raidonly ) ) then
						SKG_LF_ReservePlayerButton:Disable()
					end
				end
			end
		end
	end
end

-- Main Frame creation

SKG_Frame = CreateFrame( "Frame", "SKG_Frame", UIParent )
	SKG_Frame:SetClampedToScreen( true )
	SKG_Frame:SetHeight( 512 )
	SKG_Frame:SetWidth( 256 )
	SKG_Frame:SetPoint( "TOPLEFT", UIParent, "TOPLEFT", 100, -100 )
	SKG_Frame:EnableMouse( true )
	SKG_Frame:SetMovable( true )
	SKG_Frame:SetFrameLevel( 1 )
	SKG_Frame:SetFrameStrata( "FULLSCREEN_DIALOG" )
	SKG_Frame:SetHitRectInsets( 2, 5, 4, 4 )
	SKG_Frame:Hide()
	
	SKG_Frame:RegisterForDrag( "LeftButton" )
	SKG_Frame:SetScript( "OnDragStart",
		function(this)
			this:StartMoving()
		end
	)
	SKG_Frame:SetScript( "OnDragStop", 
		function(this)
			local db = SKG.db.profile.Options.General
			this:StopMovingOrSizing()
			db.xPos = SKG_Frame:GetLeft()
			db.yPos = SKG_Frame:GetTop()
		end
	)

	local SKG_Frame_Texture = SKG_Frame:CreateTexture( "SKG_Frame_Texture", "ARTWORK" )
		SKG_Frame_Texture:SetTexture( [[Interface\Addons\SKG\BLP\Main]] )
		SKG_Frame_Texture:SetAllPoints( SKG_Frame )

	local SKG_Frame_CloseButton = CreateFrame( "Button", "SKG_Frame_CloseButton", SKG_Frame, "UIPanelCloseButton" )
		SKG_Frame_CloseButton:SetFrameLevel( 5 )
		SKG_Frame_CloseButton:SetScript( "OnClick",
			function(this)
				this:GetParent():Hide()
			end
		)
		SKG_Frame_CloseButton:SetPoint( "TOPRIGHT", SKG_Frame, "TOPRIGHT", 5, -6 )
	
	local SKG_TitleText = SKG_Frame:CreateFontString( "SKG_TitleText", "OVERLAY", "GameFontHighlight" )
		SKG_TitleText:SetText( "-----" )
		SKG_TitleText:SetPoint( "TOP", SKG_Frame, "TOP", 20, -15 )
		SKG_TitleText:SetHeight( 14 )
		SKG_TitleText:SetWidth( 200 )

	local SKG_SelectListButton = SKG_CreateButton( "SKG_SelectListButton", SKG_Frame, 22, 170,
		"TOPLEFT", SKG_Frame, "TOPLEFT", 74, -41, "GameFontNormalSmall", "GameFontHighlightSmall", 
		SKGL["SKG_LISTBUTTON_TEXT"], "CENTER", "SKG_LISTBUTTON_TT" )

		SKG_SelectList_QTC = LibStub("LibQTipClick-1.1")
	
		--[[ Removing to use LQTC instead
		SKG_SelectListButton:SetScript( "OnClick",
			function()
				Dewdrop:Register( this,
					'children', SKG.listSelections,
					'dontHook', true,
					'point', "TOPRIGHT",
					'relativePoint', "BOTTOMRIGHT"
				)
				this:SetScript( "OnClick",
					function()
						if Dewdrop:IsOpen( this ) then
							Dewdrop:Close()
						else
							Dewdrop:Open( this )
						end
					end
				)
				this:GetScript( "OnClick" )()
			end
		)
]]--
	local SKG_OptionSwitcherButton = CreateFrame( "Button", "SKG_OptionSwitcherButton", SKG_Frame, "UIPanelButtonTemplate" )
 		SKG_OptionSwitcherButton:SetWidth( 64 )
		SKG_OptionSwitcherButton:SetHeight( 64 )
 		SKG_OptionSwitcherButton:SetPoint( "TOPLEFT", SKG_Frame, "TOPLEFT", 9, -3 )

		SKG_SetOptionSwitcherButtonTexture( 1 )

		SKG_OptionSwitcherButton:SetScript( "OnClick", SKG_FlipFrame )
		SKG_OptionSwitcherButton:SetScript( "OnEnter", 
			function ( this )
				GameTooltip_SetDefaultAnchor( GameTooltip, this )
				GameTooltip:SetText( SKGL["SKG_OPTIONBUTTON_TT"], HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b )
				GameTooltip:Show()
			end
		)
		SKG_OptionSwitcherButton:SetScript( "OnLeave",
			function( this )
				GameTooltip:Hide()
			end
		)

	local SKG_ActiveListTitle = SKG_Frame:CreateFontString( "SKG_ActiveListTitle", "OVERLAY", "GameFontHighlight" )
		SKG_ActiveListTitle:SetText( "" )
		SKG_ActiveListTitle:SetPoint( "TOP", SKG_Frame, "TOP", -8, -76 )
		SKG_ActiveListTitle:SetHeight( 14 )
		SKG_ActiveListTitle:SetWidth( 200 )

	SKG_ListItem = { }
	for i = 1,SKG_maxVisibleListItems do
			local Temp_ListItem = SKG_CreateButton("SKG_ListItem" .. i, SKG_Frame, 16, 190,
				"TOP", SKG_Frame, "TOP", 4, -93, "GameFontNormalSmall", "GameFontHighlightSmall",
				"", "LEFT", "", 1 )
			if not ( i == 1 ) then
				Temp_ListItem:SetPoint( "TOPLEFT", SKG_ListItem[i-1], "BOTTOMLEFT", 0, -1 )
			end
			Temp_ListItem:SetScript( "OnClick",
			function ()
				SKG_ItemButton_OnClick( i )
			end )

			SKG_ListItem[i] = Temp_ListItem
	end

	local SKG_ScrollFrame = CreateFrame( "ScrollFrame", "SKG_ScrollFrame", SKG_Frame, "FauxScrollFrameTemplate" )
		SKG_ScrollFrame:SetHeight( 426 )
		SKG_ScrollFrame:SetWidth( 209 )
		SKG_ScrollFrame:SetPoint( "TOPLEFT", SKG_Frame, "TOPLEFT", 13, -72 )

		SKG_ScrollFrame:SetScript( "OnVerticalScroll",
			function( self, arg1 )
				FauxScrollFrame_OnVerticalScroll( self, arg1, 16, SKG_ScrollingList_Update)
			end )

-- OnLoad stuff here
SKG_FRAME_STATE = "Hidden"
SKG_ScrollFrame.selected = 0

-- Attempt at making the main frame close-able with "Esc" key
tinsert( UISpecialFrames, "SKG_Frame" );

--[[
************************************************************************
CHANGELOG:

Date : 04/23/09
	Removed references to the now unused Security Frame
Date : 10/25/08
	Initial attempt with Ace3 instead of LibRock. Added header/footer.
	Cleaned up a couple functions and got rid of old commented out code.
************************************************************************
]]--